query BuildLogs($deploymentId: String!, $limit: Int, $startDate: DateTime, $endDate: DateTime, $filter: String) {
	buildLogs(deploymentId: $deploymentId, limit: $limit, startDate: $startDate, endDate: $endDate, filter: $filter) {
		...LogFields
	}
}

fragment LogFields on Log {
	timestamp
	message
	attributes {
		key
		value
	}
}
