query UserProjects {
  externalWorkspaces {
    id
    name
    teamId
    projects {
      id
      name
      createdAt
      updatedAt
      deletedAt
      environments {
        edges {
          node {
            id
            name
            canAccess
            serviceInstances {
              edges {
                node {
                  serviceId
                }
              }
            }
          }
        }
      }
      services {
        edges {
          node {
            id
            name
          }
        }
      }
    }
  }
  me {
    workspaces {
      id
      name
      team {
        id
      }
      projects(first: 500) {
        edges {
          node {
            id
            name
            createdAt
            updatedAt
            deletedAt
            environments {
              edges {
                node {
                  id
                  name
                  canAccess
                  serviceInstances {
                    edges {
                      node {
                        serviceId
                      }
                    }
                  }
                }
              }
            }
            services {
              edges {
                node {
                  id
                  name
                }
              }
            }
          }
        }
      }
    }
  }
}
