subscription BuildLogs($deploymentId: String!, $filter: String, $limit: Int) {
  buildLogs(deploymentId: $deploymentId, filter: $filter, limit: $limit) {
    ...LogFields
  }
}

fragment LogFields on Log {
  timestamp
  message
  attributes {
    key
    value
  }
}
