query VariablesForServiceDeployment(
	$projectId: String!
	$environmentId: String!
	$serviceId: String!
) {
	variablesForServiceDeployment(
		projectId: $projectId
		environmentId: $environmentId
		serviceId: $serviceId
	)
}
