query Deployments($input: DeploymentListInput!, $first: Int) {
  deployments(input: $input, first: $first) {
    edges {
      node {
        id
        createdAt
        status
        meta
      }
    }
  }
}
