mutation ProjectCreate($name: String, $description: String, $workspaceId: String) {
	projectCreate(
		input: { name: $name, description: $description, workspaceId: $workspaceId }
	) {
		name
		id
		environments {
			edges {
				node {
					id
					name
				}
			}
		}
	}
}
