query Projects($workspaceId: String) {
  projects(workspaceId: $workspaceId) {
    edges {
      node {
        id
        name
        updatedAt
        workspace {
          id
          name
        }
        environments {
          edges {
            node {
              id
              name
              serviceInstances {
                edges {
                  node {
                    serviceId
                  }
                }
              }
            }
          }
        }
        services {
          edges {
            node {
              id
              name
            }
          }
        }
      }
    }
  }
}
