query DeploymentLogs($deploymentId: String!, $limit: Int, $filter: String, $startDate: DateTime, $endDate: DateTime) {
	deploymentLogs(deploymentId: $deploymentId, limit: $limit, filter: $filter, startDate: $startDate, endDate: $endDate) {
		...LogFields
	}
}

fragment LogFields on Log {
	timestamp
	message
	attributes {
		key
		value
	}
}
