query Domains(
  $environmentId: String!
  $projectId: String!
  $serviceId: String!
) {
  domains(
    environmentId: $environmentId
    projectId: $projectId
    serviceId: $serviceId
  ) {
    serviceDomains {
      id
      domain
    }
    customDomains {
      id
      domain
    }
  }
}
