query Project($id: String!) {
  project(id: $id) {
    id
    name
    deletedAt
    workspace {
      name
    }
    environments {
      edges {
        node {
          id
          name
          canAccess
          deletedAt
          serviceInstances {
            edges {
              node {
                id
                serviceId
                serviceName
                environmentId
                latestDeployment {
                  canRedeploy
                  id
                  meta
                  status
                  createdAt
                  deploymentStopped
                }
                activeDeployments {
                  id
                  status
                  createdAt
                  meta
                }
                source {
                  repo
                  image
                }
                domains {
                  serviceDomains {
                    id
                    domain
                    targetPort
                  }
                  customDomains {
                    id
                    domain
                    targetPort
                  }
                }
                cronSchedule
                nextCronRunAt
                startCommand
              }
            }
          }
          volumeInstances {
            edges {
              node {
                serviceId
                mountPath
                environmentId
                currentSizeMB
                sizeMB
                volume {
                  name
                  id
                }
              }
            }
          }
        }
      }
    }
    services {
      edges {
        node {
          id
          name
        }
      }
    }
  }
}